@@ -4,6 +4,41 @@ import pandas as pd
 import numpy as np
 
 
+def xs_weight(data):
+    pos_long = sum(data['POS'] == 1)
+    pos_short = sum(data['POS'] == -1)
+    data['WEIGHT'] = data['POS'].apply(
+        lambda x: 0.5 / pos_long if x == 1 else (0.5 / pos_short if x == -1 else 0)
+    )
+    return data
+
+
+def ts_weight(data):
+    if sum(data['POS'] != 0) > 0:
+        data['WEIGHT'] = 1 / sum(data['POS'] != 0)
+    else:
+        data['WEIGHT'] = 0
+
+    return data
+
+
+def ts_pos(data, direction=1):
+    data['POS'] = data['FACTORVALUE'] / data['FACTORVALUE'].abs() * direction
+    data = data[~np.isnan(data['POS'])].reset_index(drop=True)
+    return data
+
+
+def xs_pos(data, direction=1, quantile=50):
+    # data = data[~np.isnan(data['FACTORVALUE'])].reset_index(drop=True)
+    quantile_p = data['FACTORVALUE'].quantile(quantile / 100)
+    quantile_n = data['FACTORVALUE'].quantile(1 - quantile / 100)
+    data['POS'] = data['FACTORVALUE'].apply(
+        lambda x: direction if x > quantile_p else (-direction if x < quantile_n else np.nan)
+    )
+    data = data[~np.isnan(data['POS'])].reset_index(drop=True)
+    return data
+
+
 def tsmom(start_date, direction=1, **kwargs):
     data = kwargs['data']
     liq_days = kwargs['liq_days']
@@ -42,8 +77,9 @@ def tsmom(start_date, direction=1, **kwargs):
         )
 
         data_t['FACTORVALUE'] = data_t['CLOSE'] / data_t['CLOSE0'] - 1
-        data_t['POS'] = data_t['FACTORVALUE'] / data_t['FACTORVALUE'].abs() * direction
-        data_t['POS'] = data_t['POS'].fillna(0)
+        # data_t['POS'] = data_t['FACTORVALUE'] / data_t['FACTORVALUE'].abs() * direction
+        # data_t['POS'] = data_t['POS'].fillna(0)
+        data_t = ts_pos(data_t, direction=direction)
 
         data_tomorrow = data[np.array(data['TDATE'] == calendar[i + 1])]
 
@@ -56,10 +92,9 @@ def tsmom(start_date, direction=1, **kwargs):
 
         data_t['TDATE'] = calendar[i + 1]
         data_t['FACTOR'] = index_name
-        if sum(data_t['POS'] != 0) > 0:
-            data_t['WEIGHT'] = 1 / sum(data_t['POS'] != 0)
-        else:
-            data_t['WEIGHT'] = 0
+
+        data_t = ts_weight(data_t)
+
         positions = pd.concat([positions, data_t])
         print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
     if len(positions) == 0:
@@ -102,8 +137,9 @@ def tswr(start_date, price_field='CLOSE', lag=1, **kwargs):
             on='UCODE', how='left'
         )
         data_wr_range['FACTORVALUE'] = data_wr_range['WR1'] - data_wr_range['WR0']
-        data_wr_range['POS'] = data_wr_range['FACTORVALUE'] / data_wr_range['FACTORVALUE'].abs() * -1
-        data_wr_range['POS'] = data_wr_range['POS'].fillna(0)
+        # data_wr_range['POS'] = data_wr_range['FACTORVALUE'] / data_wr_range['FACTORVALUE'].abs() * -1
+        # data_wr_range['POS'] = data_wr_range['POS'].fillna(0)
+        data_wr_range = ts_pos(data=data_wr_range, direction=-1)
 
         data_t = data[
             np.array(data['TDATE'] == t_date)
@@ -132,10 +168,7 @@ def tswr(start_date, price_field='CLOSE', lag=1, **kwargs):
 
         data_t['TDATE'] = calendar[i + 1]
         data_t['FACTOR'] = index_name
-        if sum(data_t['POS'] != 0) > 0:
-            data_t['WEIGHT'] = 1 / sum(data_t['POS'] != 0)
-        else:
-            data_t['WEIGHT'] = 0
+        data_t = ts_weight(data_t)
         positions = pd.concat([positions, data_t])
         print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
     if len(positions) == 0:
@@ -180,24 +213,238 @@ def carry(start_date, **kwargs):
         data_t = pd.merge(data_liq, data_t, on='UCODE', how='inner')
 
         data_t = pd.merge(data_t.rename(columns={'RY': 'RY0'}), data_ry, on='UCODE')
+        data_t = data_t.rename(columns={'RY': 'FACTORVALUE'})
         data_tomorrow = data[
             np.array(data['TDATE'] == calendar[i + 1])
         ]
 
-        quantile_p = data_t['RY'].quantile(quantile / 100)
-        quantile_n = data_t['RY'].quantile(1 - quantile / 100)
+        # quantile_p = data_t['RY'].quantile(quantile / 100)
+        # quantile_n = data_t['RY'].quantile(1 - quantile / 100)
+        # data_t['POS'] = data_t['RY'].apply(lambda x: 1 if x > quantile_p else (-1 if x < quantile_n else 0))
+        data_t = xs_pos(data=data_t, quantile=quantile)
 
-        data_t['POS'] = data_t['RY'].apply(lambda x: 1 if x > quantile_p else (-1 if x < quantile_n else 0))
         data_t = pd.merge(
             data_t, data_tomorrow[['UCODE', 'CLOSE']].rename(columns={'CLOSE': 'CLOSE1'}), on='UCODE', how='left'
         )
         data_t['RETURN'] = (data_t['CLOSE1'] / data_t['CLOSE'] - 1)
-        pos_long = sum(data_t['POS'] == 1)
-        pos_short = sum(data_t['POS'] == -1)
-        data_t['WEIGHT'] = data_t['POS'].apply(
-            lambda x: 0.5 / pos_long if x == 1 else (0.5 / pos_short if x == -1 else 0)
+        data_t = xs_weight(data_t)
+        pos = data_t
+
+        pos['TDATE'] = calendar[i + 1]
+        pos['FACTOR'] = index_name
+
+        positions = pd.concat([positions, pos])
+        print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
+    if len(positions) == 0:
+        return
+
+    positions['WEIGHT'] = positions['WEIGHT'] * 100
+    positions['RETURN'] = positions['RETURN'] * 100
+    positions['FACTORVALUE'] = positions['FACTORVALUE'] * 100
+    return positions[
+        ['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CLOSE', 'CLOSE1', 'POS', 'RETURN', 'FACTORVALUE', 'FACTOR', 'WEIGHT']
+    ].reset_index(drop=True)
+
+
+def tsbasismom(start_date, **kwargs):
+    data = kwargs['data']
+    liq_days = kwargs['liq_days']
+    data['UCODE'] = data['EXCHANGE'].apply(lambda x: str(x)) + data['PCODE'].apply(lambda x: str(x))
+    calendar = data['TDATE'].drop_duplicates().tolist()
+
+    window_days = kwargs['window_days']
+    index_name = 'tsbasismom_d' + str(window_days)
+
+    start_index = calendar.index(np.array(calendar)[np.array(calendar) >= start_date][0])
+
+    positions = pd.DataFrame()
+    for i in range(start_index, len(calendar) - 1):
+        t_date = calendar[i]
+        lookback_date = calendar[i - window_days]
+
+        data_t = data[
+            np.array(data['TDATE'] == t_date)
+            & np.array(data['VOL'] > kwargs['min_volume'])
+            ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CCODE', 'CCODE2', ]]
+
+        data_liq = data[
+            data['TDATE'] == calendar[i - liq_days]
+            ][['UCODE']]
+
+        data_t = pd.merge(data_liq, data_t, on='UCODE', how='inner')
+
+        data_t_1 = kwargs['data_raw'][kwargs['data_raw']['TDATE'] == lookback_date]
+
+        data_t = data_t.merge(
+            kwargs['data_raw'][['TDATE', 'CCODE', 'CLOSE', 'LDATE']].rename(
+                columns={
+                    # 'CLOSE': 'CLOSE1t0',
+                    'LDATE': 'LDATE1',
+                }
+            ),
+            on=['TDATE', 'CCODE'],
+            how='left'
+        ).merge(
+            kwargs['data_raw'][['TDATE', 'CCODE', 'CLOSE', 'LDATE']].rename(
+                columns={
+                    'CLOSE': 'CLOSE2t0',
+                    'CCODE': 'CCODE2',
+                    'LDATE': 'LDATE2',
+                }
+            ),
+            on=['TDATE', 'CCODE2'],
+            how='left'
+        ).merge(
+            data_t_1[['CCODE', 'CLOSE']].rename(
+                columns={
+                    'CLOSE': 'CLOSE1t1'
+                }
+            ),
+            on=['CCODE'],
+            how='left'
+        ).merge(
+            data_t_1[['CCODE', 'CLOSE']].rename(
+                columns={
+                    'CLOSE': 'CLOSE2t1',
+                    'CCODE': 'CCODE2'
+                }
+            ),
+            on=['CCODE2'],
+            how='left'
         )
-        pos = data_t.rename(columns={'RY': 'FACTORVALUE'})
+
+        data_t = data_t.drop_duplicates(subset=['UCODE'], keep='last')
+        data_t['return1'] = (data_t['CLOSE'] - data_t['CLOSE1t1']) / data_t['CLOSE1t1']
+        data_t['return2'] = (data_t['CLOSE2t0'] - data_t['CLOSE2t1']) / data_t['CLOSE2t1']
+        data_t['FACTORVALUE'] = (
+                                        data_t['return1'] - data_t['return2']
+                                ) * (data_t['LDATE2'] - data_t['LDATE1']).apply(lambda x: x.days)
+
+        # data_t['POS'] = data_t['FACTORVALUE'] / abs(data_t['FACTORVALUE'])
+        # data_t = data_t[~np.isnan(data_t['POS'])].reset_index(drop=True)
+        data_t = ts_pos(data_t)
+
+        data_tomorrow = kwargs['data_raw'][
+            np.array(kwargs['data_raw']['TDATE'] == calendar[i + 1])
+        ]
+        data_tomorrow = data_tomorrow.drop_duplicates(subset=['CCODE'], keep='last')
+        data_t = pd.merge(
+            data_t, data_tomorrow[['CCODE', 'CLOSE']].rename(columns={'CLOSE': 'CLOSE1'}), on='CCODE', how='left'
+        )
+        data_t['RETURN'] = (data_t['CLOSE1'] / data_t['CLOSE'] - 1)
+        data_t = ts_weight(data_t)
+        pos = data_t[
+            ['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CLOSE', 'POS', 'FACTORVALUE', 'CLOSE1', 'RETURN', 'WEIGHT']
+        ].copy()
+
+        pos['TDATE'] = calendar[i + 1]
+        pos['FACTOR'] = index_name
+
+        positions = pd.concat([positions, pos])
+        print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
+    if len(positions) == 0:
+        return
+
+    positions['WEIGHT'] = positions['WEIGHT'] * 100
+    positions['RETURN'] = positions['RETURN'] * 100
+    positions['FACTORVALUE'] = positions['FACTORVALUE'] * 100
+    return positions[
+        ['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CLOSE', 'CLOSE1', 'POS', 'RETURN', 'FACTORVALUE', 'FACTOR', 'WEIGHT']
+    ].reset_index(drop=True)
+
+
+def xsbasismom(start_date, **kwargs):
+    data = kwargs['data']
+    liq_days = kwargs['liq_days']
+    data['UCODE'] = data['EXCHANGE'].apply(lambda x: str(x)) + data['PCODE'].apply(lambda x: str(x))
+    calendar = data['TDATE'].drop_duplicates().tolist()
+
+    window_days = kwargs['window_days']
+    quantile = kwargs['quantile']
+    index_name = 'xsbasismom_d' + str(window_days) + '_q' + str(quantile)
+
+    start_index = calendar.index(np.array(calendar)[np.array(calendar) >= start_date][0])
+
+    positions = pd.DataFrame()
+    for i in range(start_index, len(calendar) - 1):
+        t_date = calendar[i]
+        lookback_date = calendar[i - window_days]
+
+        data_t = data[
+            np.array(data['TDATE'] == t_date)
+            & np.array(data['VOL'] > kwargs['min_volume'])
+            ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CCODE', 'CCODE2', ]]
+
+        data_liq = data[
+            data['TDATE'] == calendar[i - liq_days]
+            ][['UCODE']]
+
+        data_t = pd.merge(data_liq, data_t, on='UCODE', how='inner')
+
+        data_t_1 = kwargs['data_raw'][kwargs['data_raw']['TDATE'] == lookback_date]
+
+        data_t = data_t.merge(
+            kwargs['data_raw'][['TDATE', 'CCODE', 'CLOSE', 'LDATE']].rename(
+                columns={
+                    # 'CLOSE': 'CLOSE1t0',
+                    'LDATE': 'LDATE1',
+                }
+            ),
+            on=['TDATE', 'CCODE'],
+            how='left'
+        ).merge(
+            kwargs['data_raw'][['TDATE', 'CCODE', 'CLOSE', 'LDATE']].rename(
+                columns={
+                    'CLOSE': 'CLOSE2t0',
+                    'CCODE': 'CCODE2',
+                    'LDATE': 'LDATE2',
+                }
+            ),
+            on=['TDATE', 'CCODE2'],
+            how='left'
+        ).merge(
+            data_t_1[['CCODE', 'CLOSE']].rename(
+                columns={
+                    'CLOSE': 'CLOSE1t1'
+                }
+            ),
+            on=['CCODE'],
+            how='left'
+        ).merge(
+            data_t_1[['CCODE', 'CLOSE']].rename(
+                columns={
+                    'CLOSE': 'CLOSE2t1',
+                    'CCODE': 'CCODE2'
+                }
+            ),
+            on=['CCODE2'],
+            how='left'
+        )
+
+        data_t = data_t.drop_duplicates(subset=['UCODE'], keep='last')
+        data_t['return1'] = (data_t['CLOSE'] - data_t['CLOSE1t1']) / data_t['CLOSE1t1']
+        data_t['return2'] = (data_t['CLOSE2t0'] - data_t['CLOSE2t1']) / data_t['CLOSE2t1']
+        data_t['FACTORVALUE'] = (
+                                        data_t['return1'] - data_t['return2']
+                                ) * (data_t['LDATE2'] - data_t['LDATE1']).apply(lambda x: x.days)
+        data_t = data_t[~np.isinf(data_t['FACTORVALUE'])].reset_index(drop=True)
+
+        # data_t['POS'] = data_t['FACTORVALUE'] / abs(data_t['FACTORVALUE'])
+        # data_t = data_t[~np.isnan(data_t['POS'])].reset_index(drop=True)
+        data_t = xs_pos(data_t, quantile=quantile)
+
+        data_tomorrow = kwargs['data_raw'][
+            np.array(kwargs['data_raw']['TDATE'] == calendar[i + 1])
+        ]
+        data_tomorrow = data_tomorrow.drop_duplicates(subset=['CCODE'], keep='last')
+        data_t = pd.merge(
+            data_t, data_tomorrow[['CCODE', 'CLOSE']].rename(columns={'CLOSE': 'CLOSE1'}), on='CCODE', how='left'
+        )
+        data_t['RETURN'] = (data_t['CLOSE1'] / data_t['CLOSE'] - 1)
+        data_t = xs_weight(data_t)
+        pos = data_t[
+            ['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', 'CLOSE', 'POS', 'FACTORVALUE', 'CLOSE1', 'RETURN', 'WEIGHT']
+        ].copy()
 
         pos['TDATE'] = calendar[i + 1]
         pos['FACTOR'] = index_name
@@ -257,21 +504,17 @@ def xsmom(start_date, direction=1, **kwargs):
         )
         data_t['FACTORVALUE'] = data_t['CLOSE'] / data_t['CLOSE0'] - 1
 
-        quantile_p = data_t['FACTORVALUE'].quantile(quantile / 100)
-        quantile_n = data_t['FACTORVALUE'].quantile(1 - quantile / 100)
-
-        data_t['POS'] = data_t['FACTORVALUE'].apply(
-            lambda x: direction if x > quantile_p else (-direction if x < quantile_n else 0)
-        )
+        # quantile_p = data_t['FACTORVALUE'].quantile(quantile / 100)
+        # quantile_n = data_t['FACTORVALUE'].quantile(1 - quantile / 100)
+        # data_t['POS'] = data_t['FACTORVALUE'].apply(
+        #     lambda x: direction if x > quantile_p else (-direction if x < quantile_n else 0)
+        # )
+        data_t = xs_pos(data=data_t, direction=direction, quantile=quantile)
         data_t = pd.merge(
             data_t, data_tomorrow[['UCODE', 'CLOSE']].rename(columns={'CLOSE': 'CLOSE1'}), on='UCODE', how='left'
         )
         data_t['RETURN'] = (data_t['CLOSE1'] / data_t['CLOSE'] - 1)
-        pos_long = sum(data_t['POS'] == 1)
-        pos_short = sum(data_t['POS'] == -1)
-        data_t['WEIGHT'] = data_t['POS'].apply(
-            lambda x: 0.5 / pos_long if x == 1 else (0.5 / pos_short if x == -1 else 0)
-        )
+        data_t = xs_weight(data_t)
         pos = data_t
 
         pos['TDATE'] = calendar[i + 1]
@@ -322,13 +565,14 @@ def xswr(start_date, price_field='CLOSE', lag=1, **kwargs):
             data_wr_range[data_wr_range['WR0'] > 0]['WR1'] / data_wr_range[data_wr_range['WR0'] > 0]['WR0'] - 1
         )
 
-        quantile_p = data_wr_range['FACTORVALUE'].quantile(quantile / 100)
-        quantile_n = data_wr_range['FACTORVALUE'].quantile(1 - quantile / 100)
-        data_wr_range['POS'] = data_wr_range['FACTORVALUE'].apply(
-            lambda x: -1 if x > quantile_p else (1 if x < quantile_n else 0)
-        )
+        # quantile_p = data_wr_range['FACTORVALUE'].quantile(quantile / 100)
+        # quantile_n = data_wr_range['FACTORVALUE'].quantile(1 - quantile / 100)
+        # data_wr_range['POS'] = data_wr_range['FACTORVALUE'].apply(
+        #     lambda x: -1 if x > quantile_p else (1 if x < quantile_n else 0)
+        # )
+        data_wr_range = xs_pos(data=data_wr_range, direction=-1, quantile=quantile)
 
-        data_wr_range['POS'] = data_wr_range['POS'].fillna(0)
+        # data_wr_range['POS'] = data_wr_range['POS'].fillna(0)
 
         data_t = data[
             np.array(data['TDATE'] == t_date) & np.array(data['VOL'] > kwargs['min_volume'])
@@ -356,10 +600,7 @@ def xswr(start_date, price_field='CLOSE', lag=1, **kwargs):
 
         data_t['TDATE'] = calendar[i + 1]
         data_t['FACTOR'] = index_name
-        if sum(data_t['POS'] != 0) > 0:
-            data_t['WEIGHT'] = 1 / sum(data_t['POS'] != 0)
-        else:
-            data_t['WEIGHT'] = 0
+        data_t = xs_weight(data_t)
         positions = pd.concat([positions, data_t])
         print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
     if len(positions) == 0:
@@ -380,9 +621,6 @@ def mr(start_date, price_field='CLOSE', lag=1, **kwargs):
     liq_days = kwargs['liq_days']
     data['UCODE'] = data['EXCHANGE'].apply(lambda x: str(x)) + data['PCODE'].apply(lambda x: str(x))
     data_mr['UCODE'] = data_mr['EXCHANGE'].apply(lambda x: str(x)) + data_mr['PCODE'].apply(lambda x: str(x))
-    data_mr['SIDE'] = data_mr['SIDE'].apply(lambda x: 1 if x == 3 else (-1 if x == 4 else 0))
-
-    data_mr['MR'] = data_mr['MR'] * data_mr['SIDE']
 
     calendar = data['TDATE'].drop_duplicates().tolist()
 
@@ -402,9 +640,9 @@ def mr(start_date, price_field='CLOSE', lag=1, **kwargs):
         data_mr_range = data_mr_t
 
         data_mr_range['FACTORVALUE'] = data_mr_range['MR']
-        data_mr_range['POS'] = data_mr_range['FACTORVALUE'] / data_mr_range['FACTORVALUE'].abs()
-        data_mr_range['POS'] = data_mr_range['POS'].fillna(0)
-
+        # data_mr_range['POS'] = data_mr_range['FACTORVALUE'] / data_mr_range['FACTORVALUE'].abs()
+        # data_mr_range['POS'] = data_mr_range['POS'].fillna(0)
+        data_mr_range = ts_pos(data=data_mr_range)
         data_t = data[
             np.array(data['TDATE'] == t_date) & np.array(data['VOL'] > kwargs['min_volume'])
             ][['TDATE', 'EXCHANGE', 'PCODE', 'UCODE', price_field]]
@@ -432,10 +670,7 @@ def mr(start_date, price_field='CLOSE', lag=1, **kwargs):
         # index_all.append(index_all[-1] * (1 + data_t['RETURN'].mean()))
         data_t['TDATE'] = calendar[i + 1]
         data_t['FACTOR'] = index_name
-        if sum(data_t['POS'] != 0) > 0:
-            data_t['WEIGHT'] = 1 / sum(data_t['POS'] != 0)
-        else:
-            data_t['WEIGHT'] = 0
+        data_t = ts_weight(data_t)
         positions = pd.concat([positions, data_t])
         print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
     if len(positions) == 0:
@@ -456,9 +691,6 @@ def tsmr(start_date, price_field='CLOSE', lag=1, **kwargs):
     liq_days = kwargs['liq_days']
     data['UCODE'] = data['EXCHANGE'].apply(lambda x: str(x)) + data['PCODE'].apply(lambda x: str(x))
     data_mr['UCODE'] = data_mr['EXCHANGE'].apply(lambda x: str(x)) + data_mr['PCODE'].apply(lambda x: str(x))
-    data_mr['SIDE'] = data_mr['SIDE'].apply(lambda x: 1 if x == 3 else (-1 if x == 4 else 0))
-
-    data_mr['MR'] = data_mr['MR'] * data_mr['SIDE']
 
     calendar = data['TDATE'].drop_duplicates().tolist()
 
@@ -483,8 +715,9 @@ def tsmr(start_date, price_field='CLOSE', lag=1, **kwargs):
             on='UCODE', how='left'
         )
         data_mr_range['FACTORVALUE'] = data_mr_range['MR1'] - data_mr_range['MR0']
-        data_mr_range['POS'] = data_mr_range['FACTORVALUE'] / data_mr_range['FACTORVALUE'].abs()
-        data_mr_range['POS'] = data_mr_range['POS'].fillna(0)
+        # data_mr_range['POS'] = data_mr_range['FACTORVALUE'] / data_mr_range['FACTORVALUE'].abs()
+        # data_mr_range['POS'] = data_mr_range['POS'].fillna(0)
+        data_mr_range = ts_pos(data=data_mr_range)
 
         data_t = data[
             np.array(data['TDATE'] == t_date) & np.array(data['VOL'] > kwargs['min_volume'])
@@ -513,10 +746,7 @@ def tsmr(start_date, price_field='CLOSE', lag=1, **kwargs):
         # index_all.append(index_all[-1] * (1 + data_t['RETURN'].mean()))
         data_t['TDATE'] = calendar[i + 1]
         data_t['FACTOR'] = index_name
-        if sum(data_t['POS'] != 0) > 0:
-            data_t['WEIGHT'] = 1 / sum(data_t['POS'] != 0)
-        else:
-            data_t['WEIGHT'] = 0
+        data_t = ts_weight(data_t)
         positions = pd.concat([positions, data_t])
         print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
     if len(positions) == 0:
@@ -537,9 +767,6 @@ def mrchg(start_date, price_field='CLOSE', lag=1, **kwargs):
     liq_days = kwargs['liq_days']
     data['UCODE'] = data['EXCHANGE'].apply(lambda x: str(x)) + data['PCODE'].apply(lambda x: str(x))
     data_mr['UCODE'] = data_mr['EXCHANGE'].apply(lambda x: str(x)) + data_mr['PCODE'].apply(lambda x: str(x))
-    data_mr['SIDE'] = data_mr['SIDE'].apply(lambda x: 1 if x == 3 else (-1 if x == 4 else 0))
-
-    data_mr['MRCHG'] = data_mr['MRCHG'] * data_mr['SIDE']
 
     calendar = data['TDATE'].drop_duplicates().tolist()
 
@@ -558,8 +785,9 @@ def mrchg(start_date, price_field='CLOSE', lag=1, **kwargs):
         data_mr_range = data_mr_t
 
         data_mr_range['FACTORVALUE'] = data_mr_range['MRCHG']
-        data_mr_range['POS'] = data_mr_range['FACTORVALUE'] / data_mr_range['FACTORVALUE'].abs()
-        data_mr_range['POS'] = data_mr_range['POS'].fillna(0)
+        # data_mr_range['POS'] = data_mr_range['FACTORVALUE'] / data_mr_range['FACTORVALUE'].abs()
+        # data_mr_range['POS'] = data_mr_range['POS'].fillna(0)
+        data_mr_range = ts_pos(data=data_mr_range)
 
         data_t = data[
             np.array(data['TDATE'] == t_date) & np.array(data['VOL'] > kwargs['min_volume'])
@@ -588,10 +816,7 @@ def mrchg(start_date, price_field='CLOSE', lag=1, **kwargs):
         # index_all.append(index_all[-1] * (1 + data_t['RETURN'].mean()))
         data_t['TDATE'] = calendar[i + 1]
         data_t['FACTOR'] = index_name
-        if sum(data_t['POS'] != 0) > 0:
-            data_t['WEIGHT'] = 1 / sum(data_t['POS'] != 0)
-        else:
-            data_t['WEIGHT'] = 0
+        data_t = ts_weight(data_t)
         positions = pd.concat([positions, data_t])
         print(index_name + ', ' + t_date.strftime('%Y-%m-%d') + ', ' + str(sum(data_t['POS'] != 0)))
     if len(positions) == 0:
