@@ -77,12 +77,16 @@ def index_table_gen(table, sql_info=sql_info1):
 
 def index_gen(table_factor='cta_factor', table_index='cta_index', sql_path=sql_path1):
     index_table_gen(table=table_index)
-    factors = pd.read_sql_query(
-        'select distinct FACTOR from ' + table_factor, sql_path
-    )
     end_date_all = pd.read_sql_query(
         'select distinct TDATE from ' + table_factor + ' order by TDATE desc limit 1', sql_path
     )
+    factors = pd.read_sql_query('select distinct FACTOR from ' + table_factor, sql_path)
+    # factors = pd.read_sql_query(
+    #     'select FACTOR from ' + table_factor
+    #     + ' where TDATE<=' + end_date_all['TDATE'][0].strftime('%Y%m%d')
+    #     + ' group by FACTOR',
+    #     sql_path
+    # )
     if len(factors) == 0:
         print('No factor return')
         return
@@ -91,7 +95,9 @@ def index_gen(table_factor='cta_factor', table_index='cta_index', sql_path=sql_p
             # factor = 'xswr_d1_q75'
             factor = factors['FACTOR'][i]
             last_factor_index = pd.read_sql_query(
-                'select * from ' + table_index + ' where FACTOR=\'' + factor
+                'select * from ' + table_index
+                + ' where TDATE<=' + end_date_all['TDATE'][0].strftime('%Y%m%d')
+                + ' and FACTOR=\'' + factor
                 + '\' order by TDATE desc limit 1',
                 sql_path
             )
@@ -189,6 +195,14 @@ def factor_stats(data, trade_days=250, show=False):
     if len(year_slice) > 0:
         range_dates += year_slice['TDATE'].tolist()
 
+    result = {
+        '累计收益': cumulative_return,
+        '最大回撤': min(data_g['dd'][1:]),
+        '年化收益率（头尾）': annualized_return,
+        '年化收益率（平均）': annualized_return_average,
+        '波动率': sigma
+    }
+
     if show:
         print(
             '累计收益： \n\t%.2f%%' % round(cumulative_return * 100, 2)
@@ -223,11 +237,12 @@ def factor_stats(data, trade_days=250, show=False):
                                    data_g[data_g['TDATE'] == range_dates[i]]['ind'].tolist()[0]
                                    / data_g[data_g['TDATE'] == range_dates[i - 1]]['ind'].tolist()[0]
                            ) - 1
+            result[range_dates[i - 1].strftime('%Y/%m/%d') + ' - ' + range_dates[i].strftime('%Y/%m/%d')] = range_return
             print(
                 range_dates[i - 1].strftime('%Y/%m/%d') + ' - ' + range_dates[i].strftime('%Y/%m/%d')
                 + ':\t收益：%.2f%%' % round(range_return * 100, 2)
             )
-    return data_g
+    return data_g, result
 
 
 def factor_plot(data, plot_title):
