@@ -24,7 +24,7 @@ sql_l = '''(
     `JSID` bigint not null COMMENT \'恒生聚源ID\',
     `TDATE`  date not null COMMENT \'交易日期\',
     `EXCHANGE` int COMMENT \'交易所\',
-    `CCODE` bigint COMMENT \'合约代码-恒生聚源\',
+    `CCODE` int COMMENT \'合约代码-恒生聚源\',
     `PCODE` int COMMENT \'品种代码-恒生聚源\',
     `DYEAR` int COMMENT \'交割年\',
     `DMONTH` int COMMENT \'交割月\',
@@ -254,7 +254,7 @@ def hsjy_fut_wr(exchanges=None, table='hsjy_fut_wr', db_path=None, sql_info=None
         col.pop(col.index('ROW_ID'))
         df[col].to_sql(table, create_engine(db_path), if_exists='append', index=False)
         latest_jsid = df['JSID'].tolist()[-1]
-        print('\t' + str(len(df)) + ', JSID: ' + str(latest_jsid))
+        print('\tNew data: ' + str(len(df)) + ', JSID: ' + str(latest_jsid))
 
 
 def hsjy_fut_com_info(exchanges=None, table='hsjy_fut_info_c', db_path=None, sql_info=None, page_size=None):
@@ -344,7 +344,7 @@ def hsjy_fut_com_info(exchanges=None, table='hsjy_fut_info_c', db_path=None, sql
             ]
         ].to_sql(table, create_engine(db_path), if_exists='append', index=False)
         latest_jsid = df['JSID'].tolist()[-1]
-        print('\t' + str(len(df)) + ', JSID: ' + str(latest_jsid))
+        print('\tNew data: ' + str(len(df)) + ', JSID: ' + str(latest_jsid))
 
 
 def hsjy_fut_pro_info(exchanges=None, table='hsjy_fut_info_p', db_path=None, sql_info=None, page_size=None):
@@ -423,7 +423,7 @@ def hsjy_fut_pro_info(exchanges=None, table='hsjy_fut_info_p', db_path=None, sql
             ]
         ].to_sql(table, create_engine(db_path), if_exists='append', index=False)
         latest_jsid = df['JSID'].tolist()[-1]
-        print('\t' + str(len(df)) + ', JSID: ' + str(latest_jsid))
+        print('\tNew data: ' + str(len(df)) + ', JSID: ' + str(latest_jsid))
 
 
 def hsjy_fut_com(
@@ -493,7 +493,7 @@ def hsjy_fut_com(
         )
         df.to_sql(table, create_engine(db_path), if_exists='append', index=False)
         latest_jsid = df['JSID'].tolist()[-1]
-        print('\t' + str(len(df)) + ', JSID: ' + str(latest_jsid))
+        print('\tNew data: ' + str(len(df)) + ', JSID: ' + str(latest_jsid))
 
 
 def hsjy_fut_member(table='hsjy_fut_memberrank', db_path=None, sql_info=None, page_size=None):
@@ -564,7 +564,7 @@ def hsjy_fut_member(table='hsjy_fut_memberrank', db_path=None, sql_info=None, pa
         col.pop(col.index('ROW_ID'))
         df[col].to_sql(table, create_engine(db_path), if_exists='append', index=False)
         latest_jsid = df['JSID'].tolist()[-1]
-        print('\t' + str(len(df)) + ', JSID: ' + str(latest_jsid))
+        print('\tNew data: ' + str(len(df)) + ', JSID: ' + str(latest_jsid))
 
 
 if __name__ == '__main__':
