@@ -1,5 +1,5 @@
 #!/usr/bin/python
-#coding:utf-8
+# coding:utf-8
 hb_ip = '192.168.223.152'
 
 sql_user_hb = {
@@ -195,3 +195,17 @@ db_tables = {
     'hsjy_member_rank': 'HSJY_GG.FUT_MEMBERRANKBYCONTRACT',
     'ch_stocks_daily_quote': 'FINCHINA.CHDQUOTE',
 }
+
+wind_sec = {
+    'CCFI.WI': 'Wind商品指数',
+    'NMFI.WI': 'Wind贵金属',
+    'NFFI.WI': 'Wind有色',
+    'JJRI.WI': 'Wind煤焦钢矿',
+    'NMBM.WI': 'Wind非金属建材',
+    'ENFI.WI': 'Wind能源',
+    'CIFI.WI': 'Wind化工',
+    'CRFI.WI': 'Wind谷物',
+    'OOFI.WI': 'Wind油脂油料',
+    'SOFI.WI': 'Wind软商品',
+    'APFI.WI': 'Wind农副产品'
+}
