@@ -1,5 +1,5 @@
 #!/usr/bin/python
-#coding:utf-8
+# coding:utf-8
 # 统计基金表现的def
 
 import pandas as pd
@@ -154,12 +154,18 @@ def performance_analysis(
         # if len(dd_all) == 0:
         #     dd_all = single_df[['t_date', 'dd']].rename(columns={'dd': i})
         # else:
-        dd_all = dd_all.merge(single_df[[date_col, 'dd']].rename(columns={'dd': i}), on=date_col, how='left')
+        if 'dd' in single_df.columns:
+            dd_all = dd_all.merge(single_df[[date_col, 'dd']].rename(columns={'dd': i}), on=date_col, how='left')
+        else:
+            dd_all[i] = None
 
         # if len(ret_all) == 0:
         #     ret_all = single_df[['t_date', 'ret']].rename(columns={'ret': i})
         # else:
-        ret_all = ret_all.merge(single_df[[date_col, 'ret']].rename(columns={'ret': i}), on=date_col, how='left')
+        if 'ret' in single_df.columns:
+            ret_all = ret_all.merge(single_df[[date_col, 'ret']].rename(columns={'ret': i}), on=date_col, how='left')
+        else:
+            ret_all[i] = None
 
     return result_all, dd_all, ret_all
 
@@ -170,14 +176,16 @@ def ret(data_df):
     if 't_date' in funds:
         funds.remove(date_col)
     elif '日期' in funds:
-        date_col = '日期'
+        # date_col = '日期'
         funds.remove('日期')
-    result_all = data_df['t_date']
-    for i in funds:
-        single_df = data_df[[date_col, i]].copy()
-        single_df['ret'] = single_df[i] / single_df[i].shift(1) - 1
-
-        result_all = pd.merge(result_all, single_df[['t_date', 'ret']], on='t_date').rename(columns={'ret': i})
+    result_all = data_df[funds].pct_change()
+    result_all['t_date'] = data_df['t_date']
+    # result_all = data_df['t_date']
+    # for i in funds:
+    #     single_df = data_df[[date_col, i]].copy()
+    #     single_df['ret'] = single_df[i] / single_df[i].shift(1) - 1
+    #
+    #     result_all = pd.merge(result_all, single_df[['t_date', 'ret']], on='t_date').rename(columns={'ret': i})
     return result_all
 
 
@@ -446,8 +454,15 @@ def update_performance(start_d, end_d, funds, db_path, cal_db_path, month_releas
     fund_start_date = []
     for n in fund_list['name']:
         fund_name_list.append(n)
-        fund_date_list.append(funds_data[funds_data[n] > 0]['t_date'].tolist()[-1].strftime('%Y-%m-%d'))
-        fund_start_date.append(funds_data[funds_data[n] > 0]['t_date'].tolist()[0].strftime('%Y/%m/%d'))
+        try:
+            fund_date_list.append(funds_data[funds_data[n] > 0]['t_date'].tolist()[-1].strftime('%Y-%m-%d'))
+        except IndexError:
+            fund_date_list.append(None)
+
+        try:
+            fund_start_date.append(funds_data[funds_data[n] > 0]['t_date'].tolist()[0].strftime('%Y/%m/%d'))
+        except IndexError:
+            fund_start_date.append(None)
 
     result_date = pd.DataFrame(
         {
@@ -475,8 +490,15 @@ def update_performance(start_d, end_d, funds, db_path, cal_db_path, month_releas
     funds.remove('t_date')
 
     for i in funds:
-        nav_2019.loc[:, i] = nav_2019[i] / nav_2019[nav_2019[i] > 0][i].tolist()[0]
-        nav_2020.loc[:, i] = nav_2020[i] / nav_2020[nav_2020[i] > 0][i].tolist()[0]
+        try:
+            nav_2019.loc[:, i] = nav_2019[i] / nav_2019[nav_2019[i] > 0][i].tolist()[0]
+        except IndexError:
+            nav_2019.loc[:, i] = None
+
+        try:
+            nav_2020.loc[:, i] = nav_2020[i] / nav_2020[nav_2020[i] > 0][i].tolist()[0]
+        except IndexError:
+            nav_2020.loc[:, i] = None
 
     result = {
         'main': results_all,
