@@ -9,28 +9,25 @@ from sqlalchemy import create_engine
 pymysql.install_as_MySQLdb()
 
 
-def load_calendar(start_date=None, end_date=None, freq='', db_path='', table='stocks_index'):
+def load_calendar(start_date=None, end_date=None, freq='', db_path='', table='stocks_index_ts'):
     if start_date is None:
         start_date = datetime(2010, 1, 1).date()
 
     if end_date is None:
         end_date = datetime.now().date()
 
-    if freq.lower() in ['w', 'week', 'weekly']:
-        freq_end = '_weekly'
-    elif freq.lower() in ['month', 'monthly']:
-        freq_end = '_monthly'
-    else:
-        freq_end = ''
     engine_stocks = create_engine(db_path)
     trade_cal = pd.read_sql_query(
-        'select distinct t_date from '
-        + table + '_ts' + freq_end
+        'select distinct t_date from ' + table
         + ' where t_date>=' + start_date.strftime('%Y%m%d') + ' and t_date<='
         + end_date.strftime('%Y%m%d') + ' order by t_date',
         engine_stocks
     )
-    return trade_cal
+    if freq.lower() in ['w', 'week', 'weekly']:
+        trade_cal = trade_cal.set_index(pd.to_datetime(trade_cal['t_date'])).resample('W').max().reset_index(drop=True)
+    elif freq.lower() in ['m', 'month', 'monthly']:
+        trade_cal = trade_cal.set_index(pd.to_datetime(trade_cal['t_date'])).resample('M').max().reset_index(drop=True)
+    return trade_cal[trade_cal['t_date'] >= start_date].reset_index(drop=True)
 
 
 # 交易日的日期戳(周度或月度)
@@ -109,7 +106,10 @@ def load_funds_data(
         if fillna:
             # latest_date_i = fund_data[fund_data[fund_name] > 0].index[-1]
             # fund_data = fund_data.fillna(method='ffill')
-            latest_date = fund_data[fund_data[fund_name] > 0].reset_index()['t_date'].tolist()[-1]
+            try:
+                latest_date = fund_data[fund_data[fund_name] > 0].reset_index()['t_date'].tolist()[-1]
+            except IndexError:
+                latest_date = fund_data.reset_index()['t_date'].tolist()[0]
             if freq != '':
                 try:
                     latest_date_week = cal_weekly[
@@ -134,7 +134,10 @@ def load_funds_data(
                 if all_aligned:
                     # 算月度净值时先统一成周净值再转换成日净值，去掉部分日净值以同一所有产品净值频率，不直接按日净值转换为月净值
                     fund_data = pd.merge(cal_weekly, fund_data, on='t_date', how='left')
-                nav_end_date = fund_data[fund_data[fund_name] > 0]['t_date'].tolist()[-1]
+                try:
+                    nav_end_date = fund_data[fund_data[fund_name] > 0]['t_date'].tolist()[-1]
+                except IndexError:
+                    nav_end_date = fund_data['t_date'].tolist()[0]
                 fund_data = pd.merge(cal, fund_data, on='t_date', how='left')
                 fund_data = fund_data.fillna(method='ffill')
                 if cal_spe['t_date'].tolist()[-1] > nav_end_date:
@@ -154,10 +157,11 @@ def load_funds_data(
         try:
             i_dates.append(fund_data['t_date'][fund_data[fund_name] > 0].tolist()[0])
         except IndexError:
-            i_dates.append(None)
+            pass
+            # i_dates.append(np.nan)
         cal_all = pd.merge(cal_all, fund_data, on='t_date', how='left')
 
-    i_date = min(i_dates)
+    i_date = np.nanmin(i_dates)
     if i_date > first_date:
         first_date = i_date
     return cal_all[cal_all['t_date'] >= first_date].reset_index(drop=True)
