@@ -86,6 +86,11 @@ industry_cluster_dict = {"大金融": ["银行", "非银金融", "房地产"],
                          "周期": ["钢铁", "采掘", "有色金属", "化工", "建筑装饰", "建筑材料"],
                          "制造": ["公用事业", "交通运输", "机械设备", "汽车", "电气设备", "轻工制造"]}
 
+cta_factors = ['tsmom_d1', 'tsmom_d2', 'tsmom_d3', 'tsmom_d5', 'tsmom_d10', 'tsmom_d20', 'tsmom_d30',
+               'carry_d20_q70', 'tsbasismom_d5', 'tsbasismom_d20']
+
+cta_factor_map_dict = {"rev_short": "短期反转", "tendency_medium": "中短期趋势", "tendency_long": "中长期趋势",
+                       "roll_return": "展期收益", "basis_mom_short": "短期基差动量", "basis_mom_long": "中长期基差动量"}
 
 sql_params = {
     "ip": "192.168.223.152",
